part of masamune_purchase;

/// Define the billing item.
///
/// Pass this object to [PurchaseCore.initialize()] to
/// get the billing item details from the server.
///
/// By defining [onDeliver], processing at the time of billing can be added.
// ignore: must_be_immutable
@immutable
class MobilePurchaseProduct implements PurchaseProduct {
  /// Define the billing item.
  ///
  /// Pass this object to [PurchaseCore.initialize()] to
  /// get the billing item details from the server.
  ///
  /// By defining [onDeliver], processing at the time of billing can be added.
  const MobilePurchaseProduct({
    required this.id,
    this.type = ProductType.consumable,
    this.value = 0,
    this.target,
    String? defaultName,
    String? defaultText,
    double defaultPrice = 0.0,
    this.subscriptionData,
    this.isEnabledListener,
    this.isRestoreTransaction,
    this.onDeliver,
  })  : name = defaultName,
        text = defaultText,
        price = defaultPrice;

  /// Basement id.
  @override
  final String id;

  /// Product type.
  @override
  final ProductType type;

  /// Product value.
  @override
  final double value;

  /// Target path.
  @override
  final String? target;

  /// Product name.
  @override
  final String? name;

  /// Product description.
  @override
  final String? text;

  /// Product price.
  @override
  final double price;

  /// Callback to restore billing.
  ///
  /// True to restore billing.
  ///
  /// [subscriptionChecker] stores the callback that checks by passing the document for subscription.
  final Future<bool> Function(
    PurchaseDetails purchase,
    bool Function(DynamicMap document) subscriptionChecker,
  )? isRestoreTransaction;

  /// Check out if non-consumption items and subscriptions are valid.
  ///
  /// If true, billing is enabled.
  ///
  /// [subscriptionChecker] stores the callback that checks by passing the document for subscription.
  final ValueNotifier<bool>? Function(
    PurchaseProduct product,
    SubscribeOptions subscribeOptions,
    bool Function(DynamicMap document) subscriptionChecker,
  )? isEnabledListener;

  /// Callback for delivering billing items.
  final Future<void> Function(
    PurchaseDetails purchase,
    PurchaseProduct product,
    PurchaseModel core,
  )? onDeliver;

  /// Get the data to be added at the time of subscription.
  final Future<DynamicMap?> Function(
    PurchaseDetails purchase,
    PurchaseModel core,
  )? subscriptionData;

  /// Check out if non-consumption items and subscriptions are valid.
  ///
  /// If true, billing is enabled.
  @override
  bool get enabled => type == ProductType.consumable;

  /// Product Id.
  @override
  String get productId => id;

  /// The name of the item.
  @override
  String get productName => name ?? "";

  /// Item description.
  @override
  String get productText => text ?? "";

  /// The price of the item.
  @override
  String get productPrice => price.toString();
}

// ignore: must_be_immutable
class _MobilePurchaseProduct extends MobilePurchaseProduct {
  _MobilePurchaseProduct({
    required String id,
    ProductType type = ProductType.consumable,
    double value = 0.0,
    String? target,
    String? defaultName,
    String? defaultText,
    double defaultPrice = 0.0,
    Future<DynamicMap?> Function(
      PurchaseDetails purchase,
      PurchaseModel core,
    )?
        subscriptionData,
    ValueNotifier<bool>? Function(
      PurchaseProduct product,
      SubscribeOptions subscribeOptions,
      bool Function(DynamicMap document) subscriptionChecker,
    )?
        isEnabledListener,
    Future<bool> Function(
      PurchaseDetails purchase,
      bool Function(DynamicMap document) subscriptionChecker,
    )?
        isRestoreTransaction,
    Future<void> Function(
      PurchaseDetails purchase,
      PurchaseProduct product,
      PurchaseModel core,
    )?
        onDeliver,
  })  : assert(id.isNotEmpty, "The id is empty."),
        super(
          id: id,
          type: type,
          value: value,
          target: target,
          defaultName: defaultName,
          defaultPrice: defaultPrice,
          defaultText: defaultText,
          subscriptionData: subscriptionData,
          isEnabledListener: isEnabledListener,
          isRestoreTransaction: isRestoreTransaction,
          onDeliver: onDeliver,
        );

  _MobilePurchaseProduct.from(
    PurchaseProduct product, {
    Future<DynamicMap?> Function(PurchaseDetails purchase, PurchaseModel core)?
        subscriptionData,
    ValueNotifier<bool>? Function(
      PurchaseProduct product,
      SubscribeOptions subscribeOptions,
      bool Function(DynamicMap document) subscriptionChecker,
    )?
        isEnabledListener,
    Future<bool> Function(
      PurchaseDetails purchase,
      bool Function(DynamicMap document) subscriptionChecker,
    )?
        isRestoreTransaction,
    Future<void> Function(
      PurchaseDetails purchase,
      PurchaseProduct product,
      PurchaseModel core,
    )?
        onDeliver,
  }) : this(
          id: product.id,
          type: product.type,
          value: product.value,
          target: product.target,
          defaultName: product.name,
          defaultText: product.text,
          defaultPrice: product.price,
          subscriptionData: subscriptionData,
          isEnabledListener: isEnabledListener,
          isRestoreTransaction: isRestoreTransaction,
          onDeliver: onDeliver,
        );

  void _setInternal(ProductDetails details) => _productDetails = details;

  ProductDetails? _productDetails;

  @override
  bool get enabled =>
      type == ProductType.consumable || (_enabledValueNotifier?.value ?? false);

  // ignore: cancel_subscriptions
  ValueNotifier<bool>? _enabledValueNotifier;
  ChangeNotifierProvider<ValueNotifier<bool>>? _enabledValueNotifierProvider;

  /// Product Id.
  @override
  String get productId {
    if (_productDetails?.id.isEmpty ?? true) {
      return id;
    }
    return _productDetails!.id;
  }

  /// The name of the item.
  @override
  String get productName {
    if (_productDetails?.title.isEmpty ?? true) {
      return name ?? "";
    }
    return _productDetails!.title;
  }

  /// Item description.
  @override
  String get productText {
    if (_productDetails?.description.isEmpty ?? true) {
      return text ?? "";
    }
    return _productDetails!.description;
  }

  /// The price of the item.
  @override
  String get productPrice {
    if (_productDetails?.price.isEmpty ?? true) {
      return price.toString();
    }
    return _productDetails!.price;
  }
}
